@@ -1,7 +1,7 @@
-/*  $Id: xfcesmart.c 4615 2008-04-18 15:25:52Z afb $
+/*  $Id: xfcesmart.c 5802 2008-10-25 09:28:41Z afb $
  *
  *  xfce4-smartpm-plugin - a package manager applet for the xfce4 panel
- *  Copyright (c) 2007 Anders F Bjorklund <afb@users.sourceforge.net>
+ *  Copyright (c) 2008 Anders F Bjorklund <afb@users.sourceforge.net>
  *
  *  Based on: ksmarttray by Gustavo Niemeyer <niemeyer@conectiva.com>
  *            mailwatch by Brian Tarricone <bjt23@cornell.edu>,
@@ -56,28 +56,33 @@
 
 enum
 {
-  PACKAGE_MANAGER_SMART	= 0, /* use Smart Package Manager */
-  PACKAGE_MANAGER_YUM	= 1, /* use Yum+Extender (Fedora) */
-  PACKAGE_MANAGER_APT	= 2, /* use APT+Synaptic (Ubuntu) */
-  PACKAGE_MANAGER_SLAPT	= 3, /* use slapt+gslapt (Vector) */
+  PACKAGE_MANAGER_SMART     = 0, /* use Smart Package Manager */
+  PACKAGE_MANAGER_YUM       = 1, /* use Yum+Extender (Fedora) */
+  PACKAGE_MANAGER_APT       = 2, /* use APT+Synaptic (Ubuntu) */
+  PACKAGE_MANAGER_SLAPT     = 3, /* use slapt+gslapt (Vector) */
+  PACKAGE_MANAGER_PACMAN    = 4, /* use pacman+gtkpm (Arch)   */
 };
 
-#define SMART_WEBSITE "http://smartpm.org/"
-#define SMART_PROGRAM "smart" /* regular bin */
-#define SMART_UPDATE  "smart-update" /* suid */
-#define SMART_SUIDBIN "smart-root" /* Fedora */
+#define SMART_WEBSITE       "http://smartpm.org/"
+#define SMART_PROGRAM       "smart" /* regular bin */
+#define SMART_UPDATE        "smart-update" /* suid */
+#define SMART_SUIDBIN       "smart-root" /* Fedora */
 
-#define YUM_WEBSITE   "http://www.yum-extender.org/"
-#define YUM_PROGRAM   "yumex"
-#define YUM_UPDATE    "yum"
+#define YUM_WEBSITE         "http://www.yum-extender.org/"
+#define YUM_PROGRAM         "yumex"
+#define YUM_UPDATE          "yum"
 
-#define APT_WEBSITE   "http://www.nongnu.org/synaptic/"
-#define APT_PROGRAM   "synaptic"
-#define APT_UPDATE    "apt-get"
+#define APT_WEBSITE         "http://www.nongnu.org/synaptic/"
+#define APT_PROGRAM         "synaptic"
+#define APT_UPDATE          "apt-get"
 
-#define SLAPT_WEBSITE "http://software.jaos.org/#gslapt"
-#define SLAPT_PROGRAM /* "gslapt" */    PATH_GSLAPT
-#define SLAPT_UPDATE  /* "slapt-get" */ PATH_SLAPT_GET
+#define SLAPT_WEBSITE       "http://software.jaos.org/#gslapt"
+#define SLAPT_PROGRAM       /* "gslapt" */    PATH_GSLAPT
+#define SLAPT_UPDATE        /* "slapt-get" */ PATH_SLAPT_GET
+
+#define PACMAN_WEBSITE      "http://gtkpacman.berlios.de/"
+#define PACMAN_PROGRAM      "gtkpacman"
+#define PACMAN_UPDATE       "pacman"
 
 #define TIMEOUT_TIME  1000 /* milliseconds */
 
@@ -215,6 +220,9 @@ smart_update_channels_watch_func(GPid pid, gint status, gpointer user_data)
         case PACKAGE_MANAGER_SLAPT:
           /* TODO: determine number of new packages in Slapt channels */
           break;
+        case PACKAGE_MANAGER_PACMAN:
+          /* TODO: determine number of new packages in Pacman channels */
+          break;
       }
     }
     fclose(file);
@@ -229,7 +237,8 @@ smart_update_channels_watch_func(GPid pid, gint status, gpointer user_data)
     {
       if (strstr(p, "error: ") != NULL)
       {
-        p += strlen("error: ");
+        if (strlen(p) > 7 && memcmp(p, "error: ", 7) == 0)
+            p += strlen("error: ");
         msg = g_strdup(p);
         p = msg + strlen(msg) - 1;
         if (*p == '\n') *p = '\0';
@@ -338,6 +347,13 @@ smart_update_channels( SmartPlugin    *smart)
                 "--update" "\"" ;
       terminal = TRUE;
       break;
+    case PACKAGE_MANAGER_PACMAN:
+      argv[0] = PATH_SU;
+      argv[1] = "-c";
+      argv[2] = "\"" PACMAN_UPDATE " "\
+                "-Sy" "\"" ;
+      terminal = TRUE;
+      break;
     default:
       return 0;
   }
@@ -352,6 +368,7 @@ smart_update_channels( SmartPlugin    *smart)
     if (smart->package_manager == PACKAGE_MANAGER_YUM) title = YUM_UPDATE;
     if (smart->package_manager == PACKAGE_MANAGER_APT) title = APT_UPDATE;
     if (smart->package_manager == PACKAGE_MANAGER_SLAPT) title = SLAPT_UPDATE;
+    if (smart->package_manager == PACKAGE_MANAGER_PACMAN) title = PACMAN_UPDATE;
   
     args = g_strdup_printf("%s %s %s",
                  argv[0], argv[1], argv[2]);
@@ -502,6 +519,14 @@ smart_check_upgrades_watch_func(GPid pid, gint status, gpointer user_data)
               packages = atoi(p);
             /* TODO: read package names */
             break;
+          case PACKAGE_MANAGER_PACMAN:
+            if (p[0] == ':' && p[1] == ':')
+              continue;
+            if (strcmp(p, "resolving dependencies..."))
+              continue;
+            packages++;
+            /* TODO: read package names */
+            break;
        }
       }
       else
@@ -525,7 +550,7 @@ smart_check_upgrades_watch_func(GPid pid, gint status, gpointer user_data)
         continue;
       }
       if (smart->package_manager != PACKAGE_MANAGER_SMART ||
-          strstr(p, "error: ") != NULL)
+          (strstr(p, "error: ") != NULL && strstr(p, "warning: ") != NULL))
       {
         if (strstr(p, "error: ") == 0)
           p += strlen("error: ");
@@ -630,6 +655,11 @@ smart_check_upgrades( SmartPlugin    *smart)
       argv[1] = "--simulate";
       argv[2] = (smart->dist_upgrade) ? "--dist-upgrade" : "--upgrade";
       break;
+    case PACKAGE_MANAGER_PACMAN:
+      argv[0] = PACMAN_UPDATE;
+      argv[1] = "-Sup";
+      argv[1] = "--noconfirm";
+      break;
     default:
       return 0;
   }
@@ -717,6 +747,12 @@ smart_sudo_smart_gui(SmartPlugin *smart, gchar *args)
       else
         program = g_strdup(SLAPT_PROGRAM);
       break;
+    case PACKAGE_MANAGER_PACMAN:
+      if (smart_program_exists(PATH_GKSU))
+        program = g_strdup_printf("%s %s", PATH_GKSU, PACMAN_PROGRAM);
+      else
+        program = g_strdup(PACMAN_PROGRAM);
+      break;
     default:
       return;
   }
@@ -739,11 +775,33 @@ smart_sudo_smart_gui(SmartPlugin *smart, gchar *args)
 static void
 smart_launch_program(SmartPlugin *smart)
 {
+  gchar *upgrade;
+
+  switch (smart->package_manager)
+  {
+    case PACKAGE_MANAGER_SMART:
+      upgrade = "upgrade";
+      break;
+    case PACKAGE_MANAGER_YUM:
+      upgrade = NULL;
+      break;
+    case PACKAGE_MANAGER_APT:
+      upgrade = NULL;
+      break;
+    case PACKAGE_MANAGER_SLAPT:
+      upgrade = "--upgrade";
+      break;
+    case PACKAGE_MANAGER_PACMAN:
+      upgrade = NULL;
+      break;
+    default:
+      return;
+  }
+
   if (smart->upgrade_directly && smart->updates_available > 0)
-    smart_sudo_smart_gui(smart,
-      (smart->package_manager == PACKAGE_MANAGER_SLAPT) ? "--upgrade" : "upgrade");
-   else
-     smart_sudo_smart_gui(smart, NULL);
+    smart_sudo_smart_gui(smart, upgrade);
+  else
+    smart_sudo_smart_gui(smart, NULL);
 }
 
 #ifdef HAVE_LIBNOTIFY
@@ -797,9 +855,13 @@ smart_show_notification(SmartPlugin *smart, gchar *summary, gchar *message)
     notify_notification_add_action (n,
       "default", "default",
       smart_notify_callback, smart, NULL);
-    notify_notification_add_action (n,
-      "upgrade", _("Upgrade Now"),
-      smart_notify_callback, smart, NULL);
+    
+    if (smart->package_manager == PACKAGE_MANAGER_SMART)
+    {
+      notify_notification_add_action (n,
+        "upgrade", _("Upgrade Now"),
+        smart_notify_callback, smart, NULL);
+    }
 
     notify_notification_set_hint_int32 (n, "x", x);
     notify_notification_set_hint_int32 (n, "y", y);
@@ -984,6 +1046,12 @@ smart_combobox_changed_cb(GtkComboBox *combo,
 
   smart_set_manager (smart, manager);
 
+  button = g_object_get_data (G_OBJECT (smart->plugin), "upgrade-directly");
+
+  if (button != NULL)
+  gtk_widget_set_sensitive (button,
+     smart->package_manager == PACKAGE_MANAGER_SMART);
+
   button = g_object_get_data (G_OBJECT (smart->plugin), "dist-upgrade");
 
   if (button != NULL)
@@ -1218,6 +1286,9 @@ smart_size_changed (XfcePanelPlugin *plugin,
     case PACKAGE_MANAGER_SLAPT:
       iconname = "xfce4-" SLAPT_PROGRAM;
       break;
+    case PACKAGE_MANAGER_PACMAN:
+      iconname = "xfce4-" PACMAN_PROGRAM;
+      break;
     default:
       iconname = "xfce-unknown";
       break;
@@ -1313,6 +1384,9 @@ smart_configure_response (GtkWidget    *dialog,
     case PACKAGE_MANAGER_SLAPT:
       website = SLAPT_WEBSITE;
       break;
+    case PACKAGE_MANAGER_PACMAN:
+      website = PACMAN_WEBSITE;
+      break;
     default:
       website = "http://goodies.xfce.org/";
       break;
@@ -1345,6 +1419,7 @@ smart_configure_response (GtkWidget    *dialog,
     }
   else if (response == GTK_RESPONSE_OK)
     {
+      g_object_set_data (G_OBJECT (smart->plugin), "upgrade-directly", NULL);
       g_object_set_data (G_OBJECT (smart->plugin), "dist-upgrade", NULL);
 
       /* remove the dialog data from the plugin */
@@ -1453,6 +1528,7 @@ smart_configure (XfcePanelPlugin *plugin,
   gtk_combo_box_append_text (GTK_COMBO_BOX(combo), _("Yum+Extender"));
   gtk_combo_box_append_text (GTK_COMBO_BOX(combo), _("APT+Synaptic"));
   gtk_combo_box_append_text (GTK_COMBO_BOX(combo), _("slapt+gslapt"));
+  gtk_combo_box_append_text (GTK_COMBO_BOX(combo), _("pacman+gtkpm"));
   gtk_combo_box_set_active (GTK_COMBO_BOX(combo), smart->package_manager);
   gtk_widget_show (combo);
   g_signal_connect (G_OBJECT(combo),"changed", G_CALLBACK(smart_combobox_changed_cb), smart);
@@ -1501,6 +1577,11 @@ smart_configure (XfcePanelPlugin *plugin,
   g_signal_connect (G_OBJECT(button),"toggled", G_CALLBACK(smart_checkbox4_toggle_cb), smart);
   gtk_box_pack_start (GTK_BOX(vbox),button,TRUE,TRUE,0);
 
+  g_object_set_data (G_OBJECT (plugin), "upgrade-directly", button);
+  gtk_widget_set_sensitive (button,
+     smart->package_manager == PACKAGE_MANAGER_SMART ||
+     smart->package_manager == PACKAGE_MANAGER_SLAPT);
+
   button = gtk_check_button_new_with_label(_("Smart upgrade to newer release (dist-upgrade)"));
   gtk_widget_show (button);
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(button), smart->dist_upgrade);
@@ -1541,7 +1622,7 @@ smart_about (XfcePanelPlugin *plugin)
               _("Smart Package Manager - panel plugin"),
               VERSION,
               _("Check for updates using the Smart Package Manager"),
-              XFCE_COPYRIGHT_TEXT("2007", "Anders F Bjorklund"),
+              XFCE_COPYRIGHT_TEXT("2007-2008", "Anders F Bjorklund"),
               XFCE_LICENSE_GPL);
   xfce_about_info_set_homepage(info, "http://goodies.xfce.org/projects/panel-plugins/xfce4-smartpm-plugin");
 
